package org.unix4j.unix;

import org.unix4j.command.CommandInterface;

import org.unix4j.unix.sed.SedFactory;

/**
 * Non-instantiable module with inner types making up the <b>sed</b> command.
 * <p>
 * <b>NAME</b>
 * <p>
 * sed - stream editor for filtering and transforming text 
 * <p>
 * <b>SYNOPSIS</b>
 * <p>
 * <table>
 * <tr><td width="10px"></td><td nowrap="nowrap">{@code sed <args>}</td></tr>
 * <tr><td width="10px"></td><td nowrap="nowrap">{@code sed <script>}</td></tr>
 * </table>
 * <p>
 * See {@link Interface} for the corresponding command signature methods.
 * <p>
 * <b>DESCRIPTION</b>
 * <p>
 * <p>	Sed is a stream editor. A stream editor is used to perform basic text	transformations on an input stream (a file or input from a pipeline).	While in some ways similar to an editor which permits scripted edits	(such as ed), sed works by making only one pass over the input(s),	and is consequently more efficient. But it is sed's ability to filter	text in a pipeline which particularly distinguishes it from other	types of editors.</p><b>NOTES</b><p>	<ul>		<li>Currently only the substitute "s" command is supported.</li>		<li>Currently only the forward slash "/" can be used as the script delimiter. e.g. s/search/replace/g</li>		<li>A forward slash can be used in the search or replace expression, escaped with a backslash.</li>		<li>The g option can be used at the end of the script to denote whether substitute is replace first or replace all per line.</li>	</ul><p>	<b>Some examples:</b>	<br/>	<pre>input day into sed("s/day/night/") This will output "night"</pre>	<br/>	<pre>input "day and night" into sed("s/\\sand\\s/-to-/") This will output "day-to-night"</pre>	(Note the use of \s whitespace character).	<br/>	Java regular expressions are used for searching and replacing. For an	overview of the Java pattern matching, and substition, please see the	{@link java.util.regex.Pattern} documentation.</p>
 * 
 * <p>
 * <b>Options</b>
 * <p>
 * The command supports no options.
 * <p>
 * <b>OPERANDS</b>
 * <p>
 * The following operands are supported:
 * <p>
 * <table>
 * <tr valign="top"><td width="10px"></td><td nowrap="nowrap">{@code <script>}</td><td>&nbsp;:&nbsp;</td><td nowrap="nowrap">{@code String}</td><td>&nbsp;</td><td>Sed script</td></tr>
 * <tr valign="top"><td width="10px"></td><td nowrap="nowrap">{@code <args>}</td><td>&nbsp;:&nbsp;</td><td nowrap="nowrap">{@code String...}</td><td>&nbsp;</td><td>Sed script</td></tr>
 * </table>
 */
public final class Sed {
	/**
	 * The "sed" command name.
	 */
	public static final String NAME = "sed";

	/**
	 * Interface defining all method signatures for the "sed" command.
	 * 
	 * @param <R>
	 *            the generic return type for all command signature methods
	 *            to support different implementor types; the methods of a 
	 *            command factory for instance returns a command instance; 
	 *            command builders can also implement this interface, but their
	 *            methods return the builder itself enabling for chained method
	 *            invocation to create joined commands
	 */
	public static interface Interface<R> extends CommandInterface<R> {
		/**
		 * Executes the given sed script.
		 *
		 * @param args Sed script
		 * @return the generic type {@code <R>} defined by the implementing class;
		 *         the command itself returns no value and writes its result to the
		 *         standard output; see class level parameter comments for more 
		 *         details
		 */
		R sed(String... args);
		/**
		 * Executes the given sed script.
		 *
		 * @param script Sed script
		 * @return the generic type {@code <R>} defined by the implementing class;
		 *         the command itself returns no value and writes its result to the
		 *         standard output; see class level parameter comments for more 
		 *         details
		 */
		R sed(String script);
	}

	/**
	 * Singleton {@link SedFactory factory} instance for the "sed" command.
	 */
	public static final SedFactory Factory = SedFactory.INSTANCE;

	// no instances
	private Sed() {
		super();
	}
}
