package org.unix4j.unix.sed;

import java.util.List;
import java.util.Map;
import java.util.Arrays;

import org.unix4j.command.Arguments;
import org.unix4j.context.ExecutionContext;
import org.unix4j.convert.ValueConverter;
import org.unix4j.util.ArgsUtil;
import org.unix4j.util.ArrayUtil;
import org.unix4j.variable.Arg;
import org.unix4j.variable.VariableContext;

import org.unix4j.unix.Sed;

/**
 * Arguments and options for the {@link Sed sed} command.
 */
public final class SedArguments implements Arguments<SedArguments> {
	

	
	// operand: <script>
	private String script;
	private boolean scriptIsSet = false;
	
	// operand: <args>
	private String[] args;
	private boolean argsIsSet = false;
	
	/**
	 * Constructor to use if no options are specified.
	 */
	public SedArguments() {
		super();
	}

	/**
	 * Constructor string arguments encoding options and arguments, possibly
	 * also containing variable expressions. 
	 * 
	 * @param args string arguments for the command
	 * @throws NullPointerException if args is null
	 */
	public SedArguments(String... args) {
		this();
		this.args = args;
		this.argsIsSet = true;
	}

	private Object resolveVariable(VariableContext context, String variable) {
		final Object value = context.getValue(variable);
		if (value != null) {
			return value;
		}
		throw new IllegalArgumentException("cannot resolve variable " + variable + 
				" in command: sed " + this);
	}
	private Object[] resolveVariables(VariableContext context, String... unresolved) {
		final Object[] resolved = new Object[unresolved.length];
		for (int i = 0; i < resolved.length; i++) {
			final String expression = unresolved[i];
			if (Arg.isVariable(expression)) {
				resolved[i] = resolveVariable(context, expression);
			} else {
				resolved[i] = expression;
			}
		}
		return resolved;
	}
	private <V> V convert(ExecutionContext context, String operandName, Class<V> operandType, Object value) {
		final ValueConverter<V> converter = context.getValueConverterFor(operandType);
		final V convertedValue;
		if (converter != null) {
			convertedValue = converter.convert(value);
		} else {
			convertedValue = null;
		}
		if (convertedValue != null) {
			return convertedValue;
		}
		throw new IllegalArgumentException("cannot convert --" + operandName + 
				" value '" + value + "' into the type " + operandType.getName() + 
				" for sed command");
	}
	private <V> V convertList(ExecutionContext context, String operandName, Class<V> operandType, List<Object> values) {
		if (values.size() == 1) {
			final Object value = values.get(0);
			return convert(context, operandName, operandType, value);
		}
		return convert(context, operandName, operandType, values);
	}
	
	@Override
	public SedArguments getForContext(ExecutionContext context) {
		if (context == null) {
			throw new NullPointerException("context cannot be null");
		}
		if (!argsIsSet || args.length == 0) {
			//nothing to resolve
			return this;
		}

		//check if there is at least one variable
		boolean hasVariable = false;
		for (final String arg : args) {
			if (arg != null && arg.startsWith("$")) {
				hasVariable = true;
				break;
			}
		}
		//resolve variables
		final Object[] resolvedArgs = hasVariable ? resolveVariables(context.getVariableContext(), this.args) : this.args;
		
		//convert now
		final List<String> defaultOperands = Arrays.asList("script");
		final Map<String, List<Object>> map = ArgsUtil.parseArgs("options", defaultOperands, resolvedArgs);
		final SedArguments argsForContext = new SedArguments();
		for (final Map.Entry<String, List<Object>> e : map.entrySet()) {
			if ("script".equals(e.getKey())) {
					
				final String value = convertList(context, "script", String.class, e.getValue());  
				argsForContext.setScript(value);
			} else if ("args".equals(e.getKey())) {
				throw new IllegalStateException("invalid operand '" + e.getKey() + "' in sed command args: " + Arrays.toString(args));
			} else {
				throw new IllegalStateException("invalid operand '" + e.getKey() + "' in sed command args: " + Arrays.toString(args));
			}
		}
		return argsForContext;
	}
	
	/**
	 * Returns {@code <script>}: Sed script
	 * 
	 * @return the {@code <script>} operand value
	 * @throws IllegalStateException if this operand has never been set
	 */
	public String getScript() {
		if (scriptIsSet) {
			return script;
		}
		throw new IllegalStateException("operand has not been set: " + script);
	}

	/**
	 * Returns true if the {@code <script>} operand has been set. 
	 * <p>
	 * Note that this method returns true if {@link #setScript(String)}
	 * also if null was passed to the method.
	 * 
	 * @return	true if the setter for the {@code <script>} operand has 
	 * 			been called at least once
	 */
	public boolean isScriptSet() {
		return scriptIsSet;
	}
	/**
	 * Sets {@code <script>}: Sed script
	 * 
	 * @param script the value for the {@code <script>} operand
	 */
	public void setScript(String script) {
		this.script = script;
		this.scriptIsSet = true;
	}
	/**
	 * Returns {@code <args>}: Sed script
	 * 
	 * @return the {@code <args>} operand value
	 * @throws IllegalStateException if this operand has never been set
	 */
	public String[] getArgs() {
		if (argsIsSet) {
			return args;
		}
		throw new IllegalStateException("operand has not been set: " + args);
	}

	/**
	 * Returns true if the {@code <args>} operand has been set. 
	 * 
	 * @return	true if the setter for the {@code <args>} operand has 
	 * 			been called at least once
	 */
	public boolean isArgsSet() {
		return argsIsSet;
	}
	

	@Override
	public String toString() {
		// ok, we have options or arguments or both
		final StringBuilder sb = new StringBuilder();

		if (argsIsSet) {
			for (String arg : args) {
				sb.append(arg);
			}
		} else {
		
			// operand: <script>
			if (scriptIsSet) {
				if (sb.length() > 0) sb.append(' ');
				sb.append("--").append("script");
				sb.append(" ").append(toString(getScript()));
			}
			// operand: <args>
			if (argsIsSet) {
				if (sb.length() > 0) sb.append(' ');
				sb.append("--").append("args");
				sb.append(" ").append(toString(getArgs()));
			}
		}
		
		return sb.toString();
	}
	private static String toString(Object value) {
		if (value != null && value.getClass().isArray()) {
			return ArrayUtil.toString(value);
		}
		return String.valueOf(value);
	}
}
